/***************************************************************************
                          gamelist.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Thu Mar 29 2001
    copyright            : (C) 2001 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gamelist.h"

#include <string>

#include <qstring.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>

#include <kapplication.h>
#include <kstandarddirs.h>

extern QString debugFlag;

GameList::GameList(bool read_data)
{
    modified = false;
	if (read_data)
		(void)read();
}

GameList::~GameList()
{
    save();
}

bool GameList::valid(QString name)
{
    GameListMap::iterator pos = gamelist.find(name.data());
    if (pos == gamelist.end())
		return false;
    return pos->second.valid;
}

unsigned int GameList::size(QString name)
{
    GameListMap::iterator pos = gamelist.find(name.data());
    if (pos == gamelist.end())
		return 0;
    return pos->second.size;
}

void GameList::set(QString name, const unsigned int size, const bool valid)
{
    struct elem ee = { size, valid };
    gamelist[name.data()] = ee;
    modified = true;
}

bool GameList::read()
{
    QFile ff(locateLocal("appdata" ,"gamelist"));

    modified = false;
    gamelist.clear();
    if (!ff.exists() || !ff.open(IO_ReadOnly))
		return false;
    QTextStream tt(&ff);
    QString name;
    struct elem ee;
    QString valid;
    while(!tt.eof()) {
		tt >> name >> ws;
		tt >> ee.size >> ws;
		tt >> valid >> ws;
		ee.valid = (valid == "true") ? true : false;
		gamelist[name.data()] = ee;
    }
    ff.close();
    return true;
}

bool GameList::save()
{
    if (!modified)
		return true;
    QDir adir(locateLocal("appdata", ""));
    if (!adir.exists() && !adir.mkdir(adir.absPath())) {
		  if (!debugFlag.isEmpty())
			  fprintf(stderr,"Cannot create directory %s\n",
        adir.absPath().data());
		return false;
    }

    QFile ff(adir.absFilePath("gamelist"));
    if (!ff.open(IO_WriteOnly)) {
		if (!debugFlag.isEmpty())
			fprintf(stderr,"Cannot open gamlist file %s\n",
			        adir.absFilePath("gamelist").data());
		return false;
    }

    QTextStream tt(&ff);
    GameListMap::iterator pos;
    const char * valid;

    for (pos = gamelist.begin(); pos != gamelist.end(); ++pos) {
		valid = pos->second.valid ? "true" : "false";
		tt << pos->first.c_str() << "\t" << pos->second.size << "\t" <<
			valid << endl;
    }
    ff.close();
    modified = false;
    return true;
}
