/***************************************************************************
                          gamelist.h  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Thu Mar 29 2001
    copyright            : (C) 2001 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMELIST_H
#define GAMELIST_H


/**
   This class is responsable for reading and writing
   the gamelist for valid romsets and has helper
   functions to do this in a clean way.
   @author Jrgen Vigna
 */

#include <map>
#include <string>

class QString;

class GameList {
public: 
    GameList(bool read_data = true);
    ~GameList();
    /** return if a certain romset is valid **/
    bool valid(QString name);
    /** return the size of a certain romset **/
    unsigned int size(QString name);
    /** sets the values of a certain romset if this doesn't exist
	it generates a new instance in the map and initializes it
	with the passed values.
    **/
    void set(QString name, unsigned int size, bool valid);
    /** (re)read the gamelist from file **/
    bool read();
    /** save the actual values in vec to the gamelist file **/
    bool save();
	
private:
    struct elem {
		int size;
		bool valid;
    };
    typedef std::map<string, struct elem, greater<string> > GameListMap;
    
    GameListMap gamelist;
    bool modified;
};

#endif
