/***************************************************************************
                          kmamerun.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 12:43:32 CET 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>

// include files for QT
#include <qdir.h>
#include <qstrlist.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpixmap.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <ksimpleconfig.h>
#include <kaccel.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kmenubar.h>
#include <khelpmenu.h>

// application specific includes
#include "kmamerun.h"
#include "kmamerunview.h"
#include "kmamerundoc.h"
#include "maindialog.h"
#include "resource.h"
#include "preferences.h"

extern QString globalGameName, globalGameFullName;



KMameRunApp::KMameRunApp()
{
	config=kapp->config();

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initMenuBar();
	initToolBar();
	initStatusBar();
	initKeyAccel();
	initDocument();
	initView();
	
	readOptions();

	///////////////////////////////////////////////////////////////////
	// disable menu and toolbar items at startup
//  disableCommand(ID_FILE_RUN_GAME);
}

KMameRunApp::~KMameRunApp()
{
}

void KMameRunApp::initKeyAccel()
{
	keyAccel = new KAccel(this);
	
	// fileMenu accelerators
	keyAccel->connectItem(KStdAccel::New, this, SLOT(slotFileNewWindow()));
	keyAccel->connectItem(KStdAccel::Quit, this, SLOT(slotFileQuit()));

//	keyAccel->connectItem(KStdAccel::Help, kapp, SLOT(appHelpActivated()));
			
	keyAccel->changeMenuAccel(fileMenu, ID_FILE_QUIT, KStdAccel::Quit);

	keyAccel->readSettings();	
}

void KMameRunApp::initMenuBar()
{
KIconLoader *loader = KGlobal::iconLoader();
	///////////////////////////////////////////////////////////////////
	// MENUBAR

	///////////////////////////////////////////////////////////////////
	// menuBar entry fileMenu
	fileMenu = new QPopupMenu();
	fileMenu->insertItem(kapp->miniIcon(), i18n("New &Window"), ID_FILE_NEW_WINDOW);
	fileMenu->insertSeparator();
	fileMenu->insertItem(loader->loadIcon("rungame.xpm", KIcon::User), i18n("&Run Game"), ID_FILE_RUN_GAME);
	fileMenu->insertSeparator();
	fileMenu->insertItem(loader->loadIcon("preferences.xpm", KIcon::User), i18n("&Preferences"),
						 ID_FILE_PREFERENCES);
	fileMenu->insertItem(loader->loadIcon("checkroms.xpm", KIcon::User), i18n("&Check Romsets"),
						 ID_FILE_CHECKROMS);
	fileMenu->insertSeparator();
	fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);
	
	///////////////////////////////////////////////////////////////////
	// menuBar entry viewMenu
	viewMenu = new QPopupMenu();
	viewMenu->setCheckable(true);
	viewMenu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
	viewMenu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR);
	viewMenu->insertSeparator();
	viewMenu->insertItem(i18n("Sh&ow All"), ID_VIEW_ALL);
	viewMenu->insertItem(i18n("Show &Available"), ID_VIEW_AVAILABLE);
	viewMenu->insertItem(i18n("Show &Not Available"), ID_VIEW_NOTAVAILABLE);
	config->setGroup("General Options");
	int id = config->readNumEntry("show_flag",0) + ID_VIEW_ALL;
	viewMenu->setItemChecked(id, true);

	///////////////////////////////////////////////////////////////////
	// menuBar entry sortMenu
	sortMenu = new QPopupMenu();
	sortMenu->setCheckable(true);
	sortMenu->insertItem(i18n("&Full Name"), ID_SORT_FULLNAME);
	sortMenu->insertItem(i18n("&Name"), ID_SORT_NAME);
	sortMenu->insertItem(i18n("&Type"), ID_SORT_TYPE);
	sortMenu->insertItem(i18n("&Priority"), ID_SORT_PRIORITY);
	config->setGroup("General Options");
	lastSortid = config->readNumEntry("sort_order",0) + ID_SORT_FULLNAME;
	sortMenu->setItemChecked(lastSortid, true);

	///////////////////////////////////////////////////////////////////
	// menuBar entry helpMenu
	QString aboutstring=kapp->instanceName()+" "+VERSION+"\n\n";

    KHelpMenu * mHelpMenu = new KHelpMenu(0, KGlobal::instance()->aboutData(),
										  false);
	helpMenu = mHelpMenu->menu();
//	helpMenu = KMainWindow::helpMenu(aboutstring);

	///////////////////////////////////////////////////////////////////
	// MENUBAR CONFIGURATION
	// insert your popup menus with the according menubar entries in the order
	// they will appear later from left to right
	menuBar()->insertItem(i18n("&File"), fileMenu);
	menuBar()->insertItem(i18n("&View"), viewMenu);
	menuBar()->insertItem(i18n("&Sort"), sortMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem(i18n("&Help"), helpMenu);

	///////////////////////////////////////////////////////////////////
	// CONNECT THE MENU SLOTS WITH SIGNALS
	// for execution slots and statusbar messages

	connect(fileMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
	connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

	connect(viewMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
	connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

	connect(sortMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
	connect(sortMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

void KMameRunApp::initToolBar()
{

KIconLoader *loader = KGlobal::iconLoader();

	///////////////////////////////////////////////////////////////////
	// TOOLBAR

	toolBar()->insertButton(loader->loadIcon("rungame.xpm", KIcon::User), ID_FILE_RUN_GAME, true,
							i18n("Run Game"));
	toolBar()->insertSeparator();
	toolBar()->insertButton(QString("preferences"), ID_FILE_PREFERENCES, true,
							i18n("Preferences"));
	toolBar()->insertSeparator();
#if 0
	toolBar()->insertButton(QString("help"), ID_HELP_CONTENTS,
							SIGNAL(clicked()), kapp, SLOT(appHelpActivated()),
							true,i18n("Help"));
#endif

	///////////////////////////////////////////////////////////////////
	// INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE WITH toolBar(n)


	///////////////////////////////////////////////////////////////////
	// CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars by their according number
	// connect for invoking the slot actions
	connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)));
	// connect for the status help on holing icons pressed with the mouse button
	connect(toolBar(), SIGNAL(pressed(int)), SLOT(statusCallback(int)));

}

void KMameRunApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	// STATUSBAR
	// TODO: add your own items you need for displaying current application status.
	statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}

void KMameRunApp::initDocument()
{
	doc = new KMameRunDoc(this);
}

void KMameRunApp::initView()
{ 
    ////////////////////////////////////////////////////////////////////
    // create the main widget here that is managed by KTMainWindow's view-region and
    // connect the widget to your document to display document contents.

    view = new KMameRunView(this);
    connect(view->mdiag,SIGNAL(gameSelected()),this, SLOT(slotSetGamePref()));
    view->mdiag->setSortMode(lastSortid-ID_SORT_FULLNAME);
    doc->addView(view);
    setCentralWidget(view);
    localSetCaption();
    setMinimumSize(view->minimumSize().width(),
                   view->minimumSize().height()+
                   toolBar()->height() + menuBar()->height() + statusBar()->height());
    setGeometry(x(), y(),view->width(),
                view->height()+toolBar()->height()+menuBar()->height() + statusBar()->height());
    setFixedSize(width(),height());

    prefdiag = new Preferences();
    connect(prefdiag,SIGNAL(prefHide()),this,SLOT(slotPreferencesHide()));
    connect(prefdiag,SIGNAL(commitChanges()),this, SLOT(slotUpdateConfig()));
    connect(prefdiag,SIGNAL(prefGameChanged()),this, SLOT(slotUpdateListbox()));
}

void KMameRunApp::enableCommand(int id_)
{
	///////////////////////////////////////////////////////////////////
	// enable menu and toolbar functions by their ID's
	menuBar()->setItemEnabled(id_, true);
	toolBar()->setItemEnabled(id_, true);
}

void KMameRunApp::disableCommand(int id_)
{
	///////////////////////////////////////////////////////////////////
	// disable menu and toolbar functions by their ID's
	menuBar()->setItemEnabled(id_, false);
	toolBar()->setItemEnabled(id_, false);
}

void KMameRunApp::saveOptions()
{
// FIXME!! THIS IS ALL OBSOLETE, WITH NO DOCS ON HOW TO PORT IT
/*
	config->setGroup("General Options");
	config->writeEntry("Geometry", size());
	config->writeEntry("Show Toolbar", toolBar()->isVisible());
	config->writeEntry("Show Statusbar",statusBar()->isVisible());
	config->writeEntry("MenuBarPos", (int) menuBar()->menuBarPos());
	config->writeEntry("ToolBarPos", (int) toolBar()->barPos());
	config->writeEntry("Recent Files", recentFiles);
*/
}


void KMameRunApp::readOptions()
{
// FIXME!! THIS IS ALL OBSOLETE, WITH NO DOCS ON HOW TO PORT IT
/*	
	config->setGroup("General Options");

	// bar status settings
	bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
	viewMenu->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
	if(!bViewToolbar)
	{
		enableToolBar(KToolBar::Hide);
	}
	
	bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
	viewMenu->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
	if(!bViewStatusbar)
	{
		enableStatusBar(KStatusBar::Hide);
	}

	// bar position settings
	KMenuBar::menuPosition menuBarPos;
	menuBarPos=(KMenuBar::menuPosition) config->readNumEntry("MenuBarPos", KMenuBar::Top);

	KToolBar::BarPosition toolBarPos;
	toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);

	menuBar()->setMenuBarPos(menuBarPos);
	toolBar()->setBarPos(toolBarPos);
	
	QSize size=config->readSizeEntry("Geometry");
	if(!size.isEmpty())
	{
		resize(size);
	}
*/
}

bool KMameRunApp::queryExit()
{
	saveOptions();
	return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KMameRunApp::slotFileNewWindow()
{
	slotStatusMsg(i18n("Opening a new application window..."));
	
	KMameRunApp *new_window= new KMameRunApp();
	new_window->show();

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KMameRunApp::slotFileRunGame()
{
	slotStatusMsg(i18n("Running game..."));

	view->mdiag->RunGame(0);

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KMameRunApp::slotFilePreferences()
{
    slotStatusMsg(i18n("Opening Preferences Dialog..."));
	
    if (!prefdiag->isVisible()) {
		prefdiag->readConfig();
		prefdiag->show();
		mamecmd = prefdiag->prefGeneral.mamecmd.copy();
    }

    slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KMameRunApp::slotFileCheckroms()
{
    slotStatusMsg(i18n("Checking Romssets..."));
    view->mdiag->reloadListbox();
    slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KMameRunApp::slotPreferencesHide()
{
    if (!prefdiag->isVisible())
        return;
    prefdiag->hide();
}

void KMameRunApp::slotUpdateConfig()
{
    prefdiag->readConfig();
    QString cmd = prefdiag->prefGeneral.mamecmd.copy();

    if (mamecmd != cmd) {
		QString logdir = prefdiag->prefGeneral.logdir;
		QString filename;
		filename.sprintf("%s/kmamerun_showconfig%d.tmp",logdir.data(),
						 getpid());
		remove(filename);
        view->mdiag->reloadListbox();
		mamecmd = cmd;
    }
}

void KMameRunApp::slotUpdateListbox()
{
    view->mdiag->updateCurrentItem();
}

void KMameRunApp::slotSetGamePref()
{
    localSetCaption();
    prefdiag->readConfig();
}

void KMameRunApp::slotFileQuit()
{
	slotStatusMsg(i18n("Exiting..."));
	saveOptions();
	// close the first window, the list makes the next one the first again.
	// This ensures that queryClose() is called on each window to ask for closing
	KMainWindow* w;
	if(memberList)
	{
		for(w=memberList->first(); w; w=memberList->next())
		{
			// only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
			// the window and the application stay open.
			if(!w->close())
				break;
		}
	}	
	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KMameRunApp::slotViewToolBar()
{
	slotStatusMsg(i18n("Toggle the toolbar..."));
	///////////////////////////////////////////////////////////////////
	// turn Toolbar on or off
	if( viewMenu->isItemChecked(ID_VIEW_TOOLBAR))
	{
		viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
		toolBar()->hide();
	}
	else
	{
		viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
		toolBar()->show();
	}		

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KMameRunApp::slotViewStatusBar()
{
	slotStatusMsg(i18n("Toggle the statusbar..."));
	///////////////////////////////////////////////////////////////////
	//turn Statusbar on or off
	if( viewMenu->isItemChecked(ID_VIEW_STATUSBAR))
	{
		viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
		statusBar()->hide();
	}
	else
	{
		viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
		statusBar()->show();
	}

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KMameRunApp::slotSetShowMode(unsigned int mode)
{
    slotStatusMsg(i18n("Set the gamelist view mode..."));

    if (view->mdiag->showFlag != mode) {
		viewMenu->setItemChecked(view->mdiag->showFlag+ID_VIEW_ALL, false);
		viewMenu->setItemChecked(mode+ID_VIEW_ALL, true);
		view->mdiag->showFlag = mode;
		view->mdiag->reloadListbox(false);
    }

    config->setGroup("General Options");
    config->writeEntry("show_flag",mode);

    slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KMameRunApp::slotSetSortMode(int id_)
{
    slotStatusMsg(i18n("Set the gamelist sort mode..."));

    sortMenu->setItemChecked(lastSortid, false);
    sortMenu->setItemChecked(id_, true);
    lastSortid = id_;
    view->mdiag->setSortMode(id_ - ID_SORT_FULLNAME);

    config->setGroup("General Options");
    config->writeEntry("sort_order",id_ - ID_SORT_FULLNAME);

    slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


void KMameRunApp::slotStatusMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message permanently
	statusBar()->clear();
	statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KMameRunApp::slotStatusHelpMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message of whole statusbar temporary (text, msec)
	statusBar()->message(text, 2000);
}



void KMameRunApp::commandCallback(int id_)
{
    switch (id_)
    {
    case ID_FILE_NEW_WINDOW:
		slotFileNewWindow();
		break;

    case ID_FILE_RUN_GAME:
		slotFileRunGame();
		break;

    case ID_FILE_PREFERENCES:
		slotFilePreferences();
		break;
	
    case ID_FILE_CHECKROMS:
		slotFileCheckroms();
		break;
	
    case ID_FILE_QUIT:
		slotFileQuit();
		break;
  
    case ID_VIEW_TOOLBAR:
		slotViewToolBar();
		break;

    case ID_VIEW_STATUSBAR:
		slotViewStatusBar();
		break;

    case ID_VIEW_ALL:
    case ID_VIEW_AVAILABLE:
    case ID_VIEW_NOTAVAILABLE:
		slotSetShowMode(id_-ID_VIEW_ALL);
		break;
    case ID_SORT_FULLNAME:
    case ID_SORT_NAME:
    case ID_SORT_TYPE:
    case ID_SORT_PRIORITY:
		slotSetSortMode(id_);
		break;
    default:
		break;
    }
}

void KMameRunApp::statusCallback(int id_)
{
	switch (id_)
	{
    case ID_FILE_NEW_WINDOW:
		slotStatusHelpMsg(i18n("Opens a new application window"));
		break;

    case ID_FILE_RUN_GAME:
		slotStatusHelpMsg(i18n("Runs the actual selected game"));
		break;

    case ID_FILE_PREFERENCES:
		slotStatusHelpMsg(i18n("Opens the preferences dialog"));
		break;

    case ID_FILE_CHECKROMS:
		slotStatusHelpMsg(i18n("(Re)Checks the romsets"));
		break;

    case ID_FILE_QUIT:
		slotStatusHelpMsg(i18n("Quits the application"));
		break;

    case ID_VIEW_TOOLBAR:
		slotStatusHelpMsg(i18n("Enables/disables the toolbar"));
		break;

    case ID_VIEW_STATUSBAR:
		slotStatusHelpMsg(i18n("Enables/disables the statusbar"));
		break;

    case ID_VIEW_ALL:
		slotStatusHelpMsg(i18n("Show all possible games/romsets"));
		break;

    case ID_VIEW_AVAILABLE:
		slotStatusHelpMsg(i18n("Show only available games/romsets"));
		break;

    case ID_VIEW_NOTAVAILABLE:
		slotStatusHelpMsg(i18n("Show only non available games/romsets"));
		break;

    case ID_SORT_FULLNAME:
		slotStatusHelpMsg(i18n("Sort gamelist on full gamename"));
		break;

    case ID_SORT_NAME:
		slotStatusHelpMsg(i18n("Sort gamelist on gamename"));
		break;

    case ID_SORT_TYPE:
		slotStatusHelpMsg(i18n("Sort gamelist on game type"));
		break;

    case ID_SORT_PRIORITY:
		slotStatusHelpMsg(i18n("Sort gamelist on game priority"));
		break;

    default:
		break;
	}
}

void KMameRunApp::localSetCaption()
{
    QString caption="KMameRun ";
    if (!globalGameName.isEmpty())
		caption += "[" + globalGameName + "] ";
    QString elems;
    elems.setNum(view->mdiag->nrOfElements);
    setCaption(caption+"("+elems+")");
}
