/***************************************************************************
                          maindialog.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"

#include "maindialog.h"

#include "stdlib.h"

#include <qfile.h>
#include <qfiledlg.h>
#include <qmsgbox.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <qprogressbar.h>
#include <qlayout.h>

#include <kapplication.h>
#include <kprocess.h>
#include <kconfig.h>
#include <ksimpleconfig.h>
#include <kiconloader.h>
#include <klocale.h>

#include "preferences.h"
#include "myprocess.h"

#define MAXTYPE           5
#define DEFAULTPRIORITY   6

extern QString debugFlag, globalGameName, globalGameFullName;

QString MainDialog::m_str = QString::null;  //Used for the error and output listboxes

MainDialog::MainDialog(QWidget *parent, const char *name,
                       unsigned int, int)
        : QWidget(parent,name)
{
    initDialog();

    gamelistview->setColumnWidth(0,50);
    gamelistview->setColumnWidth(1,100);
    gamelistview->setColumnWidth(2,50);
    gamelistview->setColumnWidth(3,50);
    gamelistview->setHScrollBarMode(QScrollView::AlwaysOn);
    gamelistview->setVScrollBarMode(QScrollView::AlwaysOn);
    gamelistview->setFixedSize(370,380);
    gamelistview->setAllColumnsShowFocus(true);
    gamelistview->setSorting(0);

    il = new KIconLoader();

    KConfig * config = kapp->config();
    config->setGroup("General Options");
    showFlag = config->readNumEntry("show_flag", 0);
    if (showFlag > 2) // because of older versions
		showFlag = 0;

	gl = new GameList;
    loadListbox();

    connect (gamelistview, SIGNAL(returnPressed(QListViewItem *)),this,
             SLOT(RunGameCB(QListViewItem *)));
    connect (gamelistview, SIGNAL(doubleClicked(QListViewItem *)),this,
             SLOT(RunGameCB(QListViewItem *)));
    connect (gamelistview, SIGNAL(selectionChanged(QListViewItem *)),this,
             SLOT(gameSelectCB(QListViewItem *)));
    connect (inputFullFind, SIGNAL(textChanged(const QString&)),this,
             SLOT(findFullGame(const QString&)));
    connect (inputFullFind, SIGNAL(returnPressed()),this,
             SLOT(RunCurrentGame()));
    connect (inputFind, SIGNAL(textChanged(const QString&)),this,
             SLOT(findGame(const QString&)));
    connect (inputFind, SIGNAL(returnPressed()),this,
             SLOT(RunCurrentGame()));
}

MainDialog::~MainDialog()
{
    // save last selected game to config file
    KConfig * config=kapp->config();
    config->setGroup("General Options");
    config->writeEntry("LastSelectedGame",
					   gamelistview->currentItem()->text(1));
    QString logfile;
    Preferences prefs;
    logfile.sprintf("%s/kmamerun_si%d.log",
					prefs.prefGeneral.logdir.data(), getpid());
    remove(logfile);
    logfile.sprintf("%s/kmamerun_showconfig%d.tmp",
					prefs.prefGeneral.logdir.data(), getpid());
    remove(logfile);

	delete gl;
}

void MainDialog::RunGameCB(QListViewItem * item)
{
    RunGame(item);
}

void MainDialog::RunCurrentGame()
{
    RunGame(gamelistview->currentItem());
}


bool MainDialog::RunGame(QListViewItem * item)
{
    if (!item) {
		if (!(item = gamelistview->currentItem()))
			return false;
    }

    static int
        numgame = 0;
    QString
		mamedir,
        mamecmd,
        logdir,
        filename,
        logfile,
        cmd;
    QFile
        ff,fl;
    Preferences
        prefs;
    unsigned int
		size;

    mamecmd = prefs.prefGeneral.mamecmd;
    mamedir = prefs.prefGeneral.rompath;
    logdir = prefs.prefGeneral.logdir;
    if (mamecmd.isEmpty()) {
        QString str;
        str = i18n("M.A.M.E program command not specified!");
        QMessageBox::critical(this,"KMameRun",str.data());
        return false;
    }
    cmd = item->text(0);
    if (cmd.isEmpty()) {
		QMessageBox::critical(this,"KMameRun",i18n("No game selected"));
		return false;
    }
    gameName = item->text(1);
    numgame++;
    logfile.sprintf("%s/kmamerun%d.log",logdir.data(),getpid());
    fl.setName(logfile);
    // first look if the romset exists
    if (!(size=CheckRomsetExists(mamedir, gameName))) {
		QPixmap qpm = il->loadIcon("game_noroms.xpm", KIcon::NoGroup);
		// QPixmap qpm = il->loadIcon("game_noroms.xpm", 20, 20);
		item->setPixmap(0, qpm);
        QString str;
        str = i18n("It seems you don't own a romset for this game!\nOr is your rompath set wrong?");
        QMessageBox::warning(this,"KMameRun",str.data());
        return false;
    }
    // then check if the romset is correct
    if (!CheckRomsetValid(*gl, size, gameName)) {
		QPixmap qpm = il->loadIcon("game_wrongroms.xpm", KIcon::NoGroup );
		item->setPixmap(0, qpm);
        QString str;
        str = i18n("The selected game has an incorrect romset!\nI try to run it anyway");
        QMessageBox::warning(this,"KMameRun",str.data());
    }
    cmd = mamecmd.copy();
    cmd.replace(QRegExp("@@Game"),gameName);
    if (cmd == mamecmd) {
        QString str;
        str = i18n("No game selected or missing @@Game-option in\n"
				   "M.A.M.E programm command");
        QMessageBox::critical(this,"KMameRun",str.data());
        return false;
    }
    QString
        opt = prefs.optionString();
    if (!opt.isEmpty()) {
        cmd += " ";
        cmd += opt;
    }
    if (!debugFlag.isEmpty())
        fprintf(stderr,"CMD: %s\n",cmd.data());
    MyProcess
        *proc = new MyProcess;

    *proc << cmd.data();
    connect(proc, SIGNAL(processExited(KProcess *)),this,
            SLOT(RunGameDone(KProcess*)));
    connect(proc, SIGNAL(receivedStderr(KProcess *,char*,int)),this, 
            SLOT(ReadStderr(KProcess*,char*,int)));
    connect(proc, SIGNAL(receivedStdout(KProcess *,char*,int)),this, 
            SLOT(ReadStdout(KProcess*,char*,int)));

    bool result = proc->start(KProcess::NotifyOnExit , KProcess::All);
    if(!result){
        QString str;
        str.sprintf(i18n("Cannot start a new program\nfork() failed."));
        QMessageBox::critical(this,"KMameRun",str.data());
        if (proc) {
            if(proc->isRunning())
                proc->kill();
            delete proc;
        }
        return false;
    }
    return true;
}


void MainDialog::ReadStderr(KProcess *, char *buffer, int buflen)
{

    QString s = QString::fromLocal8Bit( buffer, buflen );
    QStringList errors = QStringList::split( "\n", s );

    if ( !m_str.isEmpty() ) {
      errors.first().prepend(m_str);
      m_str = QString::null;
    }

    if ( s.at(s.length() - 1) != '\n' ) {
      m_str = errors.last();
      errors.pop_back();
    }

    for ( QStringList::Iterator it = errors.begin() ; (it != errors.end() && (*it)); ++it ) {
         error_textedit->insert(*it);
    }

    error_textedit->scrollToBottom();
}

void MainDialog::ReadStdout(KProcess *, char *buffer, int buflen)
{

    QString s = QString::fromLocal8Bit( buffer, buflen );
    QStringList errors = QStringList::split( "\n", s );

    if ( !m_str.isEmpty() ) {
      errors.first().prepend(m_str);
      m_str = QString::null;
    }

    if ( s.at(s.length() - 1) != '\n' ) {
      m_str = errors.last();
      errors.pop_back();
    }

    for ( QStringList::Iterator it = errors.begin(); it != errors.end(); ++it ) {
         error_textedit->insert(*it);
    }

    error_textedit->scrollToBottom();
}

void MainDialog::RunGameDone(KProcess *proc)
{
    MyProcess *mproc = (MyProcess*)proc;

    if (mproc->localData())
        remove(mproc->localData());
    if (!debugFlag.isEmpty())
        printf("Called RunGameDone\n");
    if (proc) {
        if(proc->isRunning())
            proc->kill();
    }
    delete proc;
}

void MainDialog::resizeEvent (QResizeEvent *rse)
{
    QSize
        sz;
    int
        width,height,minh;

    sz = rse->size();
    minh = gamelistview->minimumSize().height();
    width = sz.width() - 20;
    height = sz.height() - gamelistview->y() - 10;
    if (height < minh)
        height = minh;
    gamelistview->setGeometry(gamelistview->x(), gamelistview->y(), width, height);
}

int MainDialog::totMinimumHeight()
{
    return gamelistview->y() + gamelistview->minimumSize().height();
}

void MainDialog::loadListbox(bool reinit)
{
    static bool
		init = true;
    QString
		selectGameName,
        mamecmd,
        logdir,
		mamedir,
        message = i18n("Error in call to M.A.M.E -listfull:"),
        filename,
        logfile,
        cmd;
    QFile
        ff,fl;
    Preferences
        prefs;

	if (reinit) {
		delete gl;
		gl = new GameList(false);
	}
    if (gamelistview->currentItem()) {
		if (!debugFlag.isEmpty())
			fprintf(stderr,"SELECTED-GAME: %s\n", selectGameName.data());
		selectGameName = gamelistview->currentItem()->text(1);
    }
    gamelistview->clear();
    mamecmd = prefs.prefGeneral.mamecmd;
    logdir = prefs.prefGeneral.logdir;
    mamedir = prefs.prefGeneral.rompath;
    if (mamecmd.isEmpty()) {
        QString str;
        str = i18n("M.A.M.E program command not specified!");
        QMessageBox::critical(this,"KMameRun",str.data());
        return;
    }
    logfile.sprintf("%s/kmamerun%d.log",logdir.data(),getpid());
    filename.sprintf("%s/kmamerun%d.tmp",logdir.data(),getpid());
    fl.setName(logfile);
    cmd.sprintf("%s -stdout-file %s -stderr-file %s >/dev/null 2>/dev/null",
                mamecmd.data(),filename.data(),logfile.data());
    cmd.replace(QRegExp("@@Game"),"-listfull");
    ff.setName(filename);
    if (!debugFlag.isEmpty())
        fprintf(stderr,"MAME-CMD: %s\n",(const char *)cmd);
    system(cmd);
    if (ff.size() <= 0) {
        if ((fl.size() > 0) && fl.open(IO_ReadOnly)) {
            QTextStream t(&fl);
            message += t.readLine();
            fl.close();
        }
        QMessageBox::warning(this,i18n("Error in call to M.A.M.E -listfull:"),
							 message);
        remove(logfile);
        remove(filename);
        return;
    }
    remove(logfile);
    if (!ff.open( IO_ReadOnly ))
        return;
    QListViewItem
		*select_vi = 0,
		*vi;
    QTextStream
        t(&ff);
    QString
        gname,
        gnamefull,
		gnametype,
		gnamepri,
        line,
        str;
    char
        buf[256];
    unsigned int
        ipos,
		romsexists;
    QProgressBar
        qpb(ff.size(),0,0,WStyle_Customize | WStyle_DialogBorder);
    qpb.setCaption(i18n("KMameRun (working...)"));
    qpb.setGeometry(mapToGlobal(QPoint(0,0)).x()+width()/2-100,
                    mapToGlobal(QPoint(0,0)).y()+height()/2-10,
                    200,20);
    qpb.show();
    gameNameLen = gameNameFullLen = 0;
    str = t.readLine(); // read first dumm line
    str = t.readLine(); // read second dummy line
    str = t.readLine();
    nrOfElements = 0;
    QPixmap qpm_exists = il->loadIcon("game_exists.xpm", KIcon::User );
    QPixmap qpm_noroms = il->loadIcon("game_noroms.xpm", KIcon::User );
    QPixmap qpm_novalid = il->loadIcon("game_wrongroms.xpm", KIcon::User );
    if (init) {
		KConfig * config=kapp->config();
		config->setGroup("General Options");
		selectGameName = config->readEntry("LastSelectedGame");
		if (!debugFlag.isEmpty())
			fprintf(stderr,"SELECTED_GAME: %s\n", selectGameName.data());
		init = false;
    }

    PrefGame prefGame();
    while (!t.eof() && !str.isEmpty()) {
        qpb.setProgress(ff.at());
        sscanf(str.data(),"%s",buf);
        gname = buf;
		// check if the romset exits
		romsexists = CheckRomsetExists(mamedir, gname);
		if ((!romsexists && (showFlag == 1)) || // all available games
			(romsexists && (showFlag == 2))) {  // all NOT available games
			str = t.readLine();
			continue;
		}
        if (gameNameLen < gname.length())
            gameNameLen = gname.length();
        ipos = str.find(' ');
        while((ipos < str.length()) && (str[ipos] == ' '))
            ++ipos;
        if (str[ipos] == '"')
            gnamefull = (str.mid(ipos+1,str.findRev('"')-ipos-1)).simplifyWhiteSpace();
        else
            gnamefull = (str.mid(ipos,str.length())).simplifyWhiteSpace();
        if (gameNameFullLen < gnamefull.length())
            gameNameFullLen = gnamefull.length();
		gnametype = prefs.prefgam->getGameTypeString(getGameType(gname));
		gnamepri.setNum(getGamePriority(gname)+1);
		vi = new QListViewItem(gamelistview, gnamefull, gname, gnametype,
							   gnamepri);
		if (romsexists) {
			if (!prefs.prefGeneral.check_romsets ||
				CheckRomsetValid(*gl, romsexists, gname))
				vi->setPixmap(0, qpm_exists);
			else
				vi->setPixmap(0, qpm_novalid);
		} else
			vi->setPixmap(0, qpm_noroms);
		if (selectGameName == gname)
			select_vi = vi;
		++nrOfElements;
        str = t.readLine();
    }
    ff.close();
    remove(filename);
    if (!select_vi)
		select_vi = gamelistview->firstChild();
    if (select_vi) {
		gamelistview->setSelected(select_vi, true);
		gamelistview->ensureItemVisible(select_vi);
		SetGamePixmap(select_vi);
		globalGameFullName = select_vi->text(0);
		globalGameName = select_vi->text(1);
    }
}

void MainDialog::setSortMode(int mode)
{
    gamelistview->setSorting(0);
    gamelistview->ensureItemVisible(gamelistview->currentItem());
    gamelistview->setSorting(mode);
    gamelistview->ensureItemVisible(gamelistview->currentItem());
}

void MainDialog::gameSelectCB(QListViewItem * item)
{
    SetGamePixmap(item);
    globalGameFullName = item->text(0);
    globalGameName = item->text(1);
    emit gameSelected();
}


void MainDialog::SetGamePixmap(QListViewItem * item)
{
    Preferences prefs;

    QString name = item->text(1);
    name = prefs.prefGeneral.screenshotdir + "/" + name + ".";
    name += prefs.prefmisc->getDefaultExt(prefs.prefMisc.default_ext);

    QPixmap qpm;
    qpm = il->loadIcon(name, KIcon::NoGroup, imagebutton->width());
    imagebutton->setPixmap(qpm);
}


void MainDialog::setPos(QLabel *lab, QLineEdit *le)
{
    int pos, diff;

    pos = lab->x() + lab->width() + 5;
    diff = pos - le->x();
    if (diff > 0)
        le->setGeometry(pos, le->y(), le->width() - diff, le->height());
}

void MainDialog::toggleLogfile()
{
#if 0
    showLogfile = !error_listbox->isVisible();
    if (showLogfile)
        error_listbox->show();
    else
        error_listbox->hide();
#endif
}

int MainDialog::getGameType(const char *cname)
{
    QString cgroup = cname;

    cgroup += "Game";
    KSimpleConfig config("kmamerunrc");
    config.setGroup(cgroup);
    if (config.hasKey("type"))
        return config.readNumEntry("type");
    return MAXTYPE;
}

int MainDialog::getGamePriority(const char *cname)
{
    QString cgroup = cname;

    cgroup += "Game";
    KSimpleConfig config("kmamerunrc");
    config.setGroup(cgroup);
    if (config.hasKey("priority"))
        return config.readNumEntry("priority");
    return DEFAULTPRIORITY;
}

unsigned int MainDialog::CheckRomsetExists(const char * romdir, const char * gname)
{
    QString fn1, fn2;

    fn1.sprintf("%s/%s.zip",romdir, gname);
    fn2.sprintf("%s/%s",romdir, gname);

    QFile qf1(fn1);
    QFile qf2(fn2);

    if (qf1.exists())
		return qf1.size();
    if (qf2.exists())
		return qf2.size();
	if (!debugFlag.isEmpty()) {
		fprintf(stdout, "INVALID(X): %s\n", gname);
	}
    return 0;
}

bool MainDialog::CheckRomsetValid(GameList & gl, unsigned int fsz,
								  const char * gname)
{
    // First let's see if we already checked this one and if
    // it's still the same romset. We do this by comparing the
    // binary sizes (not very acurate but IMO it's enough)
    if (gl.size(gname) == fsz) {
		if (!debugFlag.isEmpty()) {
			if (gl.valid(gname))
				fprintf(stdout,"VALID: %s\n", gname);
			else
				fprintf(stdout, "INVALID: %s\n", gname);
		}
		return gl.valid(gname);
    }

    Preferences prefs;
    QString logfile;
    QString filename;
    QString cmd;
    QString logdir = prefs.prefGeneral.logdir;
    QString mamecmd = prefs.prefGeneral.mamecmd;
	KShellProcess proc;

    // now check the romset if I did not return above
    filename.sprintf("%s/kmamerun_crs%d.tmp",logdir.data(),getpid());
    logfile.sprintf("%s/kmamerun_crs%d.log",logdir.data(),getpid());
    cmd.sprintf("%s -rompath %s -verifyroms -stdout-file %s %s >/dev/null 2>/dev/null;grep ' 0 correct' %s > %s",
				mamecmd.data(), prefs.prefGeneral.rompath.data(),
				filename.data(), gname,
				filename.data(),logfile.data());
    cmd.replace(QRegExp("@@Game"),"");
    if (!debugFlag.isEmpty())
        fprintf(stderr,"CMD: %s\n",cmd.data());
	proc << cmd;
    bool result = proc.start(KProcess::Block);
    if(!result) {
        QString str;
        str.sprintf(i18n("Cannot start a new program\nfork() failed."));
        QMessageBox::critical(this,"KMameRun",str.data());
		if(proc.isRunning())
			proc.kill();
        return false;
    }
    remove(filename);
    QFile fl(logfile);
    bool valid;
    if (fl.exists() && (fl.size() > 0)) {
		if (!debugFlag.isEmpty())
			fprintf(stdout,"INVALID: %s\n", gname);
		valid = false;
    } else {
		if (!debugFlag.isEmpty())
			fprintf(stdout, "VALID: %s\n", gname);
		valid = true;
    }
    remove(logfile);
    gl.set(gname, fsz, valid);
    return valid;
}


void MainDialog::findFullGame(const QString& str)
{
    QListViewItem * item = gamelistview->firstChild();
    if (!item)
		return;
    QString gname = item->text(0);
    while(item && (gname.find(str, 0, false) != 0)) {
		if ((item = item->itemBelow()))
			gname = item->text(0);
    }
    if (item) {
		gamelistview->setSelected(item, true);
		gamelistview->ensureItemVisible(item);
		SetGamePixmap(item);
		globalGameFullName = item->text(0);
		globalGameName = item->text(1);
    }
}


void MainDialog::findGame(const QString& str)
{
    QListViewItem * item = gamelistview->firstChild();
    if (!item)
		return;
    QString gname = item->text(1);
    while(item && (gname.find(str, 0, false) != 0)) {
		if ((item = item->itemBelow()))
			gname = item->text(1);
    }
    if (item) {
		gamelistview->setSelected(item, true);
		gamelistview->ensureItemVisible(item);
		SetGamePixmap(item);
		globalGameFullName = item->text(0);
		globalGameName = item->text(1);
    }
}


void MainDialog::updateCurrentItem()
{
    QListViewItem * item = gamelistview->currentItem();
    QListViewItem * newitem;
    QString gnamefull = item->text(0);
    QString gname = item->text(1);
    QString gnametype;
    QString gnamepri;

    PrefGame prefGame;

    gnametype = prefGame.getGameTypeString(getGameType(gname));
    gnamepri.setNum(getGamePriority(gname)+1);
    gamelistview->setSorting(0);
    newitem = new QListViewItem(gamelistview, gnamefull, gname, gnametype,
								gnamepri);
    if (newitem) {
		newitem->setPixmap(0, *item->pixmap(0));
		delete item;
		gamelistview->setSelected(newitem, true);
		gamelistview->ensureItemVisible(newitem);
		KConfig * config = kapp->config();
		config->setGroup("General Options");
		gamelistview->setSorting(config->readNumEntry("sort_order",0));
		gamelistview->ensureItemVisible(newitem);
    }
}


void MainDialog::reloadListbox(bool reinit)
{
    fprintf(stderr,"SF: %d\n", showFlag);
    QListViewItem * item = gamelistview->currentItem();
    if (!item) {
		item = gamelistview->firstChild();
		if (!item)
			return;
    }
    gamelistview->setSorting(0);
    loadListbox(reinit);
    KConfig * config = kapp->config();
    config->setGroup("General Options");
    gamelistview->setSorting(config->readNumEntry("sort_order",0));
    if ((item = gamelistview->currentItem())) {
		gamelistview->ensureItemVisible(item);
    }
}
