/***************************************************************************
                          prefaudio.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "prefaudio.h"

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <kstandarddirs.h>

PrefAudio::PrefAudio(QWidget *parent, const char *name)
        : QWidget(parent,name)
{
	initDialog();
}

PrefAudio::~PrefAudio()
{
}

void PrefAudio::setPrefs(struct audio_prefs *prefs)
{
    checkSound->setChecked(prefs->play_sound);
    checkUseSamples->setChecked(prefs->use_samples);
    checkFakeSound->setChecked(prefs->fake_sound);
    inputAudioDevice->validateAndSet(prefs->audio_device, 0, 0, 0);
    inputMixerDevice->validateAndSet(prefs->mixer_device, 0, 0, 0);
}


void PrefAudio::writeConfig(struct audio_prefs *, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Audio";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    config.writeEntry("play_sound",checkSound->isChecked());
    config.writeEntry("fake_sound",checkFakeSound->isChecked());
    config.writeEntry("use_samples",checkUseSamples->isChecked());
    config.writeEntry("audiodevice",inputAudioDevice->text());
    config.writeEntry("mixerdevice",inputMixerDevice->text());
}


void PrefAudio::readConfig(struct audio_prefs *prefs, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Audio";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (config.hasKey("play_sound"))
        prefs->play_sound = config.readNumEntry("play_sound");
    if (config.hasKey("fake_sound"))
        prefs->fake_sound = config.readNumEntry("fake_sound");
    if (config.hasKey("use_samples"))
        prefs->use_samples = config.readNumEntry("use_samples");
    if (config.hasKey("audiodevice"))
        prefs->audio_device = config.readEntry("audiodevice");
    if (config.hasKey("mixerdevice"))
        prefs->mixer_device = config.readEntry("mixerdevice");
}


const QString PrefAudio::optionString(struct audio_prefs *defpref, struct audio_prefs *prefs)
{
    QString
        opt;

    if (defpref->play_sound != prefs->play_sound) {
        if (prefs->play_sound)
            opt += " -sound";
        else
            opt += " -nosound";
    }
    if (defpref->fake_sound != prefs->fake_sound) {
        if (prefs->fake_sound)
            opt += " -fakesound";
    }
    if (defpref->use_samples != prefs->use_samples) {
        if (prefs->use_samples)
            opt += " -samples";
        else
            opt += " -nosamples";
    }
    if (defpref->audio_device != prefs->audio_device) {
        if (!prefs->audio_device.isEmpty())
            opt += " -audiodevice " + prefs->audio_device;
    }
    if (defpref->mixer_device != prefs->mixer_device) {
        if (!prefs->mixer_device.isEmpty())
            opt += " -mixerdevice " + prefs->mixer_device;
    }
    return opt;
}
