/***************************************************************************
                          preferences.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>

#include <qframe.h>
#include <qlayout.h>
#include <qpushbt.h>
#include <qlabel.h>
#include <qlined.h>
#include <qregexp.h>
#include <qfile.h>
#include <qmessagebox.h>

#include <kapplication.h>
#include <ktabctl.h>
#include <kfontdialog.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include "preferences.h"

extern QString debugFlag, globalGameName, globalGameFullName;

Preferences::Preferences()
{
    setCaption(i18n("M.A.M.E Preferences"));

    tabbar = new KTabCtl(this);
    tabbar->setGeometry(10,10, 430, 200);
    
    prefgen = new PrefGeneral(tabbar,"General");
    tabbar->addTab(prefgen, i18n("General"));
    
    prefvid = new PrefVideo(tabbar,"Video");
    tabbar->addTab(prefvid, i18n("Video"));
    
    prefaud = new PrefAudio(tabbar,"Audio");
    tabbar->addTab(prefaud, i18n("Audio"));
    
    prefinp = new PrefInput(tabbar,"Input");
    tabbar->addTab(prefinp, i18n("Input"));

    prefmisc = new PrefMisc(tabbar,"Misc");
    tabbar->addTab(prefmisc, i18n("Misc"));

    prefgam = new PrefGame(tabbar,"Game");
    tabbar->addTab(prefgam, i18n("Game"));

    b_default = new QPushButton(i18n("Defaults"), this);
    b_default->setFixedSize(80, 24);
    b_ok = new QPushButton(i18n("Ok"), this);
    b_ok->setFixedSize(80, 24);
    b_apply = new QPushButton(i18n("Apply"), this);
    b_apply->setFixedSize(80, 24);
    b_cancel = new QPushButton(i18n("Cancel"), this);
    b_cancel->setFixedSize(80, 24);

    QGridLayout *l_top_down = new QGridLayout(this, 2, 1, 4);
    QBoxLayout *l_buttons = new QBoxLayout(QBoxLayout::LeftToRight);
    l_top_down->addWidget(tabbar, 0, 0);
    l_top_down->setRowStretch(0, 1);
    l_top_down->addLayout(l_buttons, 1, 0);
    l_buttons->addStretch(1);
    l_buttons->addWidget(b_default);
    l_buttons->addStretch(1);
    l_buttons->addWidget(b_ok);
    l_buttons->addStretch(1);
    l_buttons->addWidget(b_apply);
    l_buttons->addStretch(1);
    l_buttons->addWidget(b_cancel);
    l_buttons->addStretch(1);
    l_top_down->activate();

    readConfig();
    
    connect(b_default, SIGNAL(clicked()), this, SLOT(setDefaults()) );
    connect(b_ok, SIGNAL(clicked()), this, SLOT(ok()) );
    connect(b_apply, SIGNAL(clicked()), this, SLOT(apply()) );
    connect(b_cancel, SIGNAL(clicked()), this, SLOT(cancel()) );

    setMinimumSize(370, 200);
    resize(450, 250);
}

Preferences::~Preferences()
{
}

void Preferences::readConfig()
{
    QString
        opt = "Global";
    
    setDefaultConfig();
    prefGame.game_name = globalGameName;
    prefGame.game_full_name = globalGameFullName;
    prefgam->readConfig(&prefGame,opt);
    if (!globalGameName.isEmpty()) {
        prefgam->readConfig(&prefGame,globalGameName);
        if (!prefGame.default_opt)
            opt = globalGameName;
    }
    prefgam->setPrefs(&prefGame);
    prefgen->readConfig(&prefGeneral,opt);
    prefgen->setPrefs(&prefGeneral);
    prefvid->readConfig(&prefVideo,opt);
    prefvid->setPrefs(&prefVideo);
    prefaud->readConfig(&prefAudio,opt);
    prefaud->setPrefs(&prefAudio);
    prefinp->readConfig(&prefInput,opt);
    prefinp->setPrefs(&prefInput);
    prefmisc->readConfig(&prefMisc,opt);
    prefmisc->setPrefs(&prefMisc);
}

void Preferences::writeConfig()
{
    QString
        opt;
    
    setDefaultConfig();
    if (prefgam->useDefaultOpt()) {
        opt = globalGameName;
        QString
            cgroup,
            aConfigName = locateLocal("config", "/kmamerunrc");
            //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";
        KSimpleConfig
            config(aConfigName);

        cgroup = opt + "General";
        config.deleteGroup(cgroup);
        cgroup = opt + "Video";
        config.deleteGroup(cgroup);
        cgroup = opt + "Audio";
        config.deleteGroup(cgroup);
        cgroup = opt + "Input";
        config.deleteGroup(cgroup);
        cgroup = opt + "Game";
        config.deleteGroup(cgroup);
        cgroup = opt + "Misc";
        config.deleteGroup(cgroup);
        opt = "Global";
    } else {
        opt = globalGameName;
    }
    prefgen->writeConfig(&prefGeneral,opt);
    prefvid->writeConfig(&prefVideo,opt);
    prefaud->writeConfig(&prefAudio,opt);
    prefinp->writeConfig(&prefInput,opt);
    prefmisc->writeConfig(&prefMisc,opt);
    if (!globalGameName.isEmpty()) {
        if (prefgam->writeConfig(&prefGame,globalGameName))
            emit prefGameChanged();
    }
}


void Preferences::writeGameConfig(QString const gname)
{
    if (gname.isEmpty())
	return;
    prefgam->writeConfig(&prefGame, gname);
}

void Preferences::setDefaults()
{
    setDefaultConfig();
    prefgen->setPrefs(&prefGeneral);
    prefvid->setPrefs(&prefVideo);
    prefaud->setPrefs(&prefAudio);
    prefinp->setPrefs(&prefInput);
    prefmisc->setPrefs(&prefMisc);
}

void Preferences::ok()
{
    writeConfig();
    emit commitChanges();
    emit prefHide();
}

void Preferences::apply()
{
    writeConfig();
    emit commitChanges();
}

void Preferences::cancel()
{
    emit prefHide();
}

void Preferences::closeEvent(QCloseEvent *)
{
    emit prefHide();
}

void Preferences::setPos(QLabel *lab, QLineEdit *le)
{
    int pos, diff;

    pos = lab->x() + lab->width() + 5;
    diff = pos - le->x();
    if (diff > 0)
        le->setGeometry(pos, le->y(), le->width() - diff, le->height());
}

void Preferences::setDefaultConfig()
{
	// we initialize this on the first call and then reuse it for all
	// subsequent calls this spares us the call to xmame which is really
	// unneccessary in one instance.
	static
		QStringList xmameDefaults;
	
    QString
        mamecmd,
        logdir,
        message = i18n("Error in call to M.A.M.E -showconfig:\n"),
        filename,
        cmd;
    QFile
        ff;

    //
    // preferences General
    //
    prefGeneral.mamecmd = "";
    prefGeneral.logdir = "";
    prefgen->readConfig(&prefGeneral,"Global");
    if (prefGeneral.mamecmd.isEmpty())
        prefGeneral.mamecmd = "xmame @@Game";
    if (prefGeneral.logdir.isEmpty())
        prefGeneral.logdir = "/tmp";
    mamecmd = prefGeneral.mamecmd;
    logdir = prefGeneral.logdir;
    prefGeneral.spooldir = "";
    prefGeneral.cheat = 0;
    prefGeneral.version = 37;
    prefGeneral.check_romsets = false;
    //
    // preferences Video
    //
    prefVideo.X11 = true;
    prefVideo.display = getenv("DISPLAY");
    prefVideo.heightscale = 1;
    prefVideo.frameskip = 0;
    prefVideo.use_xsync = 1;
    prefVideo.use_dirty = 1;
    prefVideo.use_mitshm = 0;
    prefVideo.private_colormap = 0;
    prefVideo.use_scanlines = 0;
    prefVideo.throttle = 1;
    prefVideo.rotation = 0;
    prefVideo.ror90 = 0;
    prefVideo.rol90 = 0;
    prefVideo.flip_Xaxis = 0;
    prefVideo.flip_Yaxis = 0;
    //
    // preferences Audio
    //
    prefAudio.play_sound = 0;
    prefAudio.fake_sound = 1;
    prefAudio.use_samples = 0;
    prefAudio.audio_device = "";
    prefAudio.mixer_device = "";
    //
    // preferences Input
    //
    prefInput.X11 = true;
    prefInput.use_mouse = 0;
    prefInput.grab_mouse = 0;
    prefInput.joystick_type = 0;
    prefInput.analogstick = 0;
    prefInput.towns_pad_name = "/dev/pad00";
    //
    // preferences Misc
    //
    prefMisc.default_ext = 0;
    prefMisc.extra_options = "";
    //
    // preferences Game
    //
    prefGame.priority = 6;
    prefGame.type = 5;
    prefGame.default_opt = true;

    if (mamecmd.isEmpty()) {
        QString str;
        str = i18n("M.A.M.E program command not specified!");
        QMessageBox::critical(this,"KMameRun",str.data());
        return;
    }

	if (xmameDefaults.empty()) {
		filename.sprintf("%s/kmamerun_showconfig%d.tmp",logdir.data(),getpid());
		cmd.sprintf("%s -stdout-file %s >/dev/null 2>/dev/null",mamecmd.data(),
					filename.data());
		cmd.replace(QRegExp("@@Game"),"-showconfig");
		ff.setName(filename);
		if (!ff.exists()) {
			if (!debugFlag.isEmpty())
				fprintf(stderr,"%s\n",cmd.data());
			system(cmd);
		}
		if (ff.size() <= 0) {
			QMessageBox::warning(this,i18n("Error in calling MAME -showconfig"),message);
			remove(filename);
			return;
		}
		if (!ff.open(IO_ReadOnly))
			return;
		
		QTextStream t(&ff);
		QString line;

		line = t.readLine(); // read first line
		while (!t.eof()) {
			if (line.left(1) == "#") {
				line = t.readLine();
				continue;
			}
			xmameDefaults.append(line);
			line = t.readLine();
		}
		ff.close();
		remove(filename);
	}

	QString
		line,
		str;
	char
		buf[256];
	int
		b;
	QStringList::Iterator it = xmameDefaults.begin();
	QStringList::Iterator itend = xmameDefaults.end();
	
	for (; it != itend; ++it) {
		line = *it;
        sscanf(line.data(),"%s",buf);
        str = buf;
        *buf = 0;
        if (str == "rompath") { // for xmame>0.34.1
            sscanf(line.data(),"%*s%s",buf);
            prefGeneral.rompath = buf;
            prefGeneral.version = 34;
        } else if (str == "spooldir") {
            sscanf(line.data(),"%*s%s",buf);
            prefGeneral.spooldir = buf;
        } else if (str == "screenshotdir") {
            sscanf(line.data(),"%*s%s",buf);
            prefGeneral.screenshotdir = buf;
        } else if (str == "mamecmd") {
            sscanf(line.data(),"%*s%s",buf);
            prefGeneral.mamecmd = buf;
        } else if (str == "display") {
            sscanf(line.data(),"%*s%s",buf);
            prefVideo.display = buf;
        } else if (str == "heightscale") {
            sscanf(line.data(),"%*s%d",&(prefVideo.heightscale));
            prefVideo.heightscale--;
        } else if (str == "brightness") {
            sscanf(line.data(),"%*s%d",&(prefVideo.brightness));
        } else if (str == "frameskip") {
            sscanf(line.data(),"%*s%d",&(prefVideo.frameskip));
        } else if (str == "xsync") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.use_xsync = b;
        } else if (str == "dirty") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.use_dirty = b;
        } else if (str == "mitshm") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.use_mitshm = b;
        } else if (str == "privatecmap") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.private_colormap = b;
        } else if (str == "scanlines") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.use_scanlines = b;
        } else if (str == "artwork") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.use_artwork = b;
        } else if (str == "throttle") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.throttle = b;
        } else if (str == "run-in-root-window") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.run_root_window = b;
        } else if (str == "x11-mode") {
            sscanf(line.data(),"%*s%d",&b);
            prefVideo.run_full_screen = b;
		} else if (str == "fullscreen") {
			sscanf(line.data(), "%*s%d", &b);
			prefVideo.run_full_screen = b;
			prefVideo.X11 = prefInput.X11 = false;
        } else if (str == "sound") {
            sscanf(line.data(),"%*s%d",&(prefAudio.play_sound));
        } else if (str == "samples") {
            sscanf(line.data(),"%*s%d",&(prefAudio.use_samples));
        } else if (str == "audiodevice") {
            sscanf(line.data(),"%*s%s",buf);
            prefAudio.audio_device = buf;
        } else if (str == "mixerdevice") {
            sscanf(line.data(),"%*s%s",buf);
            prefAudio.mixer_device = buf;
        } else if (str == "mouse") {
            sscanf(line.data(),"%*s%d",&(prefInput.use_mouse));
        } else if (str == "grabmouse") {
            sscanf(line.data(),"%*s%d",&(prefInput.grab_mouse));
        } else if (str == "joytype") {
            sscanf(line.data(),"%*s%d",&(prefInput.joystick_type));
        } else if (str == "analogstick") {
            sscanf(line.data(),"%*s%d",&(prefInput.analogstick));
        } else if (str == "towns_pad_name") {
            sscanf(line.data(),"%*s%s",buf);
            prefInput.towns_pad_name = buf;
        }

	}
	defPrefGeneral = prefGeneral;
    defPrefMisc = prefMisc;
    defPrefGame = prefGame;
    defPrefVideo = prefVideo;
    defPrefAudio = prefAudio;
    defPrefInput = prefInput;
}

const QString Preferences::optionString(const char *cname)
{
    QString
        opt;

    if (!cname)
        cname = "Global";
    
    opt += prefgen->optionString(&defPrefGeneral, &prefGeneral);
    opt += " ";
    opt += prefvid->optionString(&defPrefVideo, &prefVideo);
    opt += " ";
    opt += prefinp->optionString(&defPrefInput, &prefInput);
    opt += " ";
    opt += prefaud->optionString(&defPrefAudio, &prefAudio);
    opt += " ";
    opt += prefmisc->optionString(&defPrefMisc, &prefMisc);

    return opt;
}
