/***************************************************************************
                          prefgame.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefgame.h"

#include <qcombobox.h>

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>
#include <klocale.h>

PrefGame::PrefGame(QWidget *parent, const char *name)
        : QWidget(parent,name)
{
    initDialog();
    comboType->clear();
    comboType->setAutoResize(true);
    for(int i=0; i < 6; ++i)
	comboType->insertItem(getGameTypeString(i));
    labelGameName->setAutoResize(true);
    labelGameName->setText("Pacman");
}

PrefGame::~PrefGame()
{
}

void PrefGame::setPrefs(struct game_prefs *prefs)
{
    checkDefaultOpt->setChecked(prefs->default_opt);
    if (prefs->priority > (comboPriority->count()-1))
		prefs->priority = 6;
    comboPriority->setCurrentItem(prefs->priority);
    if (prefs->type > (comboType->count()-1))
		prefs->type = comboType->count()-1;
    comboType->setCurrentItem(prefs->type);
    labelGameName->setText(prefs->game_full_name);
    oldPriority = prefs->priority;
    oldType = prefs->type;
}

bool PrefGame::writeConfig(struct game_prefs *prefs, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Game";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (prefs->default_opt != checkDefaultOpt->isChecked())
        config.writeEntry("default_opt",checkDefaultOpt->isChecked());
    else
        config.deleteEntry("default_opt", false);
    if (prefs->priority != comboPriority->currentItem())
        config.writeEntry("priority",comboPriority->currentItem());
    else
        config.deleteEntry("priority", false);
    if (prefs->type != comboType->currentItem())
        config.writeEntry("type",comboType->currentItem());
    else
        config.deleteEntry("type", false);

    if ((oldPriority != comboPriority->currentItem()) ||
        (oldType != comboType->currentItem()))
        return true;
    return false;
}

void PrefGame::readConfig(struct game_prefs *prefs, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Game";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (config.hasKey("default_opt"))
        prefs->default_opt = config.readNumEntry("default_opt");
    if (config.hasKey("priority"))
        prefs->priority = config.readNumEntry("priority");
    if (config.hasKey("type"))
        prefs->type = config.readNumEntry("type");
}

bool PrefGame::useDefaultOpt()
{
    return checkDefaultOpt->isChecked();
}

QString PrefGame::getGameTypeString(int type)
{
	QString gType;
    switch(type) {
    case 0:
	gType = i18n("Action"); break;
    case 1:
	gType = i18n("Jump&Run"); break;
    case 2:
	gType = i18n("Shoot'em-up"); break;
    case 3:
	gType = i18n("Sport"); break;
    case 4:
	gType = i18n("Education"); break;
    default:
	gType = i18n("Other"); break;
    }
	return gType;
}
