/***************************************************************************
                          prefgeneral.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "prefgeneral.h"

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>

PrefGeneral::PrefGeneral(QWidget *parent, const char *name)
        : QWidget(parent,name)
{
	initDialog();
}

PrefGeneral::~PrefGeneral()
{
}

void PrefGeneral::setPrefs(struct general_prefs *prefs)
{
    checkCheat->setChecked(prefs->cheat);
    inputMamedir->setText(prefs->rompath);
    inputSpooldir->setText(prefs->spooldir);
    inputMamecmd->setText(prefs->mamecmd);
    inputLogdir->setText(prefs->logdir);
    inputImagedir->setText(prefs->screenshotdir);
    checkRomsets->setChecked(prefs->check_romsets);
}

void PrefGeneral::writeConfig(struct general_prefs *, const char *cname)
{
    QString
        str,
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "General";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    config.writeEntry("cheat",checkCheat->isChecked());
    str = inputMamedir->text();
    if (!str.isEmpty())
        config.writeEntry("rompath",inputMamedir->text());
    config.writeEntry("spooldir",inputSpooldir->text());
    QString
        cmd = inputMamecmd->text();
    if (!cmd.isEmpty() && cmd.find("@@Game") < 0) {
        cmd += " @@Game";
        inputMamecmd->setText(cmd);
    }
    if (cmd.isEmpty())
        cmd = "xmame @@Game";
    inputMamecmd->setText(cmd.data());
    config.writeEntry("mamecmd",inputMamecmd->text());
    config.writeEntry("logdir",inputLogdir->text());
    config.writeEntry("screenshotdir", inputImagedir->text());
    config.writeEntry("check_romsets",checkRomsets->isChecked());
}


void PrefGeneral::readConfig(struct general_prefs *prefs, const char *cname)
{
    QString
        str,
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "General";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (config.hasKey("cheat"))
        prefs->cheat = config.readNumEntry("cheat");
    if (config.hasKey("rompath")) {
        str = config.readEntry("rompath");
        if (!str.isEmpty())
            prefs->rompath = str;
    }
    // compatibility with 0.3.0 version
    else if (config.hasKey("mamedir")) {
        str = config.readEntry("mamedir");
        if (!str.isEmpty())
            prefs->rompath = str;
    }
    if (config.hasKey("spooldir"))
        prefs->spooldir = config.readEntry("spooldir");
    if (config.hasKey("mamecmd"))
        prefs->mamecmd = config.readEntry("mamecmd");
    if (config.hasKey("logdir"))
        prefs->logdir = config.readEntry("logdir");
    if (config.hasKey("screenshotdir"))
	prefs->screenshotdir = config.readEntry("screenshotdir");
    if (config.hasKey("check_romsets"))
        prefs->check_romsets = config.readNumEntry("check_romsets");
}


const QString PrefGeneral::optionString(struct general_prefs *defpref, struct general_prefs *prefs)
{
    QString
        opt;

    if (defpref->cheat != prefs->cheat) {
        if (prefs->cheat)
            opt += " -cheat";
    }
    if (defpref->rompath != prefs->rompath) {
	if (!prefs->rompath.isEmpty()) {
	    if (prefs->version > 33)
		opt += " -rompath ";
	    else
		opt += " -rompath ";
	    opt += prefs->rompath;
	}
    }
    if (defpref->spooldir != prefs->spooldir) {
	if (!prefs->spooldir.isEmpty())
            opt += " -spooldir " + prefs->spooldir;
    }
    if (defpref->screenshotdir != prefs->screenshotdir) {
	if (!prefs->screenshotdir.isEmpty())
            opt += " -screenshotdir " + prefs->screenshotdir;
    }
    return opt;
}
