/***************************************************************************
                          prefinput.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "prefinput.h"

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <kstandarddirs.h>

PrefInput::PrefInput(QWidget *parent, const char *name)
        : QWidget(parent,name)
{
	initDialog();
}

PrefInput::~PrefInput()
{
}

void PrefInput::setPrefs(struct input_prefs *prefs)
{
    comboJoyType->setCurrentItem(prefs->joystick_type);
    checkMouse->setChecked(prefs->use_mouse);
    checkGrabMouse->setChecked(prefs->grab_mouse);
    inputPadDevice->setText(prefs->towns_pad_name);
    checkAnalogStick->setChecked(prefs->analogstick);
}


void PrefInput::writeConfig(struct input_prefs *, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Input";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    config.writeEntry("joystick_type",comboJoyType->currentItem());
    config.writeEntry("towns_pad_name",inputPadDevice->text());
    config.writeEntry("use_stick_analog", checkAnalogStick->isChecked());
    config.writeEntry("use_mouse",checkMouse->isChecked());
    config.writeEntry("grab_mouse",checkGrabMouse->isChecked());
}


void PrefInput::readConfig(struct input_prefs *prefs, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Input";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (config.hasKey("joystick_type"))
        prefs->joystick_type = config.readNumEntry("joystick_type");
    if (config.hasKey("towns_pad_name"))
        prefs->towns_pad_name = config.readEntry("towns_pad_name");
    if (config.hasKey("use_stick_analog"))
        prefs->analogstick = config.readNumEntry("use_stick_analog");
    if (config.hasKey("use_mouse"))
        prefs->use_mouse = config.readNumEntry("use_mouse");
    if (config.hasKey("grab_mouse"))
        prefs->grab_mouse = config.readNumEntry("grab_mouse");
}


const QString PrefInput::optionString(struct input_prefs *defpref, struct input_prefs *prefs)
{
    QString
        opt;

    if (defpref->joystick_type != prefs->joystick_type) {
	QString type;
	type.setNum(prefs->joystick_type);
	opt += " -joytype " + type;
    }
    if (defpref->towns_pad_name != prefs->towns_pad_name) {
	if (!prefs->towns_pad_name.isEmpty() && (prefs->joystick_type == 2))
	    opt += " -paddevname " + prefs->towns_pad_name;
    }
    if (defpref->analogstick != prefs->analogstick) {
        if (prefs->analogstick)
            opt += " -analogstick";
        else
            opt += " -noanalogstick";
    }
    if (defpref->use_mouse != prefs->use_mouse) {
        if (prefs->use_mouse)
            opt += " -mouse";
        else
            opt += " -nomouse";
    }
    if (prefs->X11) {
	if (defpref->grab_mouse != prefs->grab_mouse) {
	    if (prefs->grab_mouse)
		opt += " -grabmouse";
	    else
		opt += " -nograbmouse";
	}
    }
    return opt;
}
