/***************************************************************************
                          prefvideo.cpp  -  KDE frontend for M.A.M.E
                             -------------------
    begin                : Sun Jan 23 2000
    copyright            : (C) 2000 by Jrgen Vigna
    email                : jug@sad.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "prefvideo.h"

#include <kapplication.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>

PrefVideo::PrefVideo(QWidget *parent, const char *name)
        : QWidget(parent,name)
{
    initDialog();
}

PrefVideo::~PrefVideo()
{
}

void PrefVideo::setPrefs(struct video_prefs *prefs)
{
    if (prefs->X11) {
	checkSharedMem->setChecked(prefs->use_mitshm);
	checkRefresh->setChecked(prefs->use_xsync);
	checkRunRootWindow->setChecked(prefs->run_root_window);
	checkPrivateColmap->setChecked(prefs->private_colormap);
    } else {
	checkSharedMem->setChecked(false);
	checkRefresh->setChecked(false);
	checkRunRootWindow->setChecked(false);
	checkPrivateColmap->setChecked(false);
    }
    checkRefresh->setEnabled(prefs->X11);
    checkSharedMem->setEnabled(prefs->X11);
    checkRunRootWindow->setEnabled(prefs->X11);
    checkPrivateColmap->setEnabled(prefs->X11);

    checkDirtyRec->setChecked(prefs->use_dirty);
    checkUseScanlines->setChecked(prefs->use_scanlines);
    checkUseArtwork->setChecked(prefs->use_artwork);
    checkThrottle->setChecked(prefs->throttle);
    checkNoRotate->setChecked(prefs->rotation);
    checkRotate90Right->setChecked(prefs->ror90);
    checkRotate90Left->setChecked(prefs->rol90);
    checkFlipXAxis->setChecked(prefs->flip_Xaxis);
    checkFlipYAxis->setChecked(prefs->flip_Yaxis);
    checkFullScreen->setChecked(prefs->run_full_screen);
    comboScale->setCurrentItem(prefs->heightscale);
    comboFrameskip->setCurrentItem(prefs->frameskip);
    sboxBrightness->setValue(prefs->brightness);
}

void PrefVideo::writeConfig(struct video_prefs *, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
      //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Video";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);

    config.writeEntry("use_mitshm",checkSharedMem->isChecked());
    config.writeEntry("use_xsync",checkRefresh->isChecked());
    config.writeEntry("use_dirty",checkDirtyRec->isChecked());
    config.writeEntry("private_colormap",checkPrivateColmap->isChecked());
    config.writeEntry("use_scanlines",checkUseScanlines->isChecked());
    config.writeEntry("use_artwork",checkUseArtwork->isChecked());
    config.writeEntry("enable_throttle",checkThrottle->isChecked());
    config.writeEntry("rotation",checkNoRotate->isChecked());
    config.writeEntry("ror90",checkRotate90Right->isChecked());
    config.writeEntry("rol90",checkRotate90Left->isChecked());
    config.writeEntry("flip_Xaxis",checkFlipXAxis->isChecked());
    config.writeEntry("flip_Yaxis",checkFlipYAxis->isChecked());
    config.writeEntry("run_root_window",checkRunRootWindow->isChecked());
    config.writeEntry("run_full_screen",checkFullScreen->isChecked());
    config.writeEntry("heightscale",comboScale->currentItem());
    config.writeEntry("frameskip",comboFrameskip->currentItem());
    config.writeEntry("brightness",sboxBrightness->cleanText());
}

void PrefVideo::readConfig(struct video_prefs *prefs, const char *cname)
{
    QString
        cgroup = cname,
        aConfigName = locateLocal("config", "/kmamerunrc");
        //aConfigName = KApplication::localconfigdir() + "/kmamerunrc";

    cgroup += "Video";
    KSimpleConfig config(aConfigName);
    config.setGroup(cgroup);
    if (config.hasKey("use_mitshm"))
        prefs->use_mitshm = config.readNumEntry("use_mitshm");
    if (config.hasKey("use_xsync"))
        prefs->use_xsync = config.readNumEntry("use_xsync");
    if (config.hasKey("use_dirty"))
        prefs->use_dirty = config.readNumEntry("use_dirty");
    if (config.hasKey("private_colormap"))
        prefs->private_colormap = config.readNumEntry("private_colormap");
    if (config.hasKey("use_scanlines"))
        prefs->use_scanlines = config.readNumEntry("use_scanlines");
    if (config.hasKey("use_artwork"))
        prefs->use_artwork = config.readNumEntry("use_artwork");
    if (config.hasKey("enable_throttle"))
        prefs->throttle = config.readNumEntry("enable_throttle");
    if (config.hasKey("rotation"))
        prefs->rotation = config.readNumEntry("rotation");
    if (config.hasKey("ror90"))
        prefs->ror90 = config.readNumEntry("ror90");
    if (config.hasKey("rol90"))
        prefs->rol90 = config.readNumEntry("rol90");
    if (config.hasKey("flip_Xaxis"))
        prefs->flip_Xaxis = config.readNumEntry("flip_Xaxis");
    if (config.hasKey("flip_Yaxis"))
        prefs->flip_Yaxis = config.readNumEntry("flip_Yaxis");
    if (config.hasKey("run_root_window"))
        prefs->run_root_window = config.readNumEntry("run_root_window");
    if (config.hasKey("run_full_screen"))
        prefs->run_full_screen = config.readNumEntry("run_full_screen");
    if (config.hasKey("heightscale"))
        prefs->heightscale = config.readNumEntry("heightscale");
    if (config.hasKey("frameskip"))
        prefs->frameskip = config.readNumEntry("frameskip");
    if (config.hasKey("brightness"))
        prefs->brightness = config.readNumEntry("brightness");
}

const QString PrefVideo::optionString(struct video_prefs *defpref, struct video_prefs *prefs)
{
    QString opt;

    if (prefs->X11) {
	if (defpref->use_mitshm != prefs->use_mitshm) {
	    if (prefs->use_mitshm)
		opt += " -mitshm";
	    else
		opt += " -nomitshm";
	}
	if (defpref->use_xsync != prefs->use_xsync) {
	    if (prefs->use_xsync)
		opt += " -xsync";
	    else
		opt += " -noxsync";
	}
	if (defpref->private_colormap != prefs->private_colormap) {
	    if (prefs->private_colormap)
		opt += " -privatecmap";
	    else
		opt += " -noprivatecmap";
	}
	if (defpref->run_root_window != prefs->run_root_window) {
	    if (prefs->run_root_window)
		opt += " -run-in-root-window";
	    else
		opt += " -norun-in-root-window";
	}
	if (defpref->run_full_screen != prefs->run_full_screen) {
	    if (prefs->run_full_screen)
		opt += " -x11-mode 1";
	    else
		opt += " -x11-mode 0";
	}
    } else {
	if (defpref->run_full_screen != prefs->run_full_screen) {
	    if (prefs->run_full_screen)
		opt += " -fullscreen";
	    else
		opt += " -nofullscreen";
	}
    }
    if (defpref->use_dirty != prefs->use_dirty) {
        if (prefs->use_dirty)
            opt += " -dirty";
        else
            opt += " -nodirty";
    }
    if (defpref->use_scanlines != prefs->use_scanlines) {
        if (prefs->use_scanlines)
            opt += " -scanlines";
        else
            opt += " -noscanlines";
    }
    if (defpref->use_artwork != prefs->use_artwork) {
        if (prefs->use_artwork)
            opt += " -artwork";
        else
            opt += " -noartwork";
    }
    if (defpref->throttle != prefs->throttle) {
        if (prefs->throttle)
            opt += " -throttle";
        else
            opt += " -nothrottle";
    }
    if (defpref->rotation != prefs->rotation) {
        if (prefs->rotation)
            opt += " -norotate";
    }
    if (defpref->ror90 != prefs->ror90) {
        if (prefs->ror90)
            opt += " -ror";
    }
    if (defpref->rol90 != prefs->rol90) {
        if (prefs->rol90)
            opt += " -rol";
    }
    if (defpref->flip_Xaxis != prefs->flip_Xaxis) {
        if (prefs->flip_Xaxis)
            opt += " -flipx";
    }
    if (defpref->flip_Yaxis != prefs->flip_Yaxis) {
        if (prefs->flip_Yaxis)
            opt += " -flipy";
    }
    if (defpref->heightscale != prefs->heightscale) {
        QString
            s;
        s.sprintf("%d",prefs->heightscale+1);
        opt += " -scale " + s;
    }
    if (defpref->frameskip != prefs->frameskip) {
        QString
            s;
        s.sprintf("%d",prefs->frameskip);
        opt += " -frameskip " + s;
    }
    if (defpref->brightness != prefs->brightness) {
        QString
            s;
        s.sprintf("%d",prefs->brightness);
        opt += " -brightness " + s;
    }
    return opt;
}
